import { Type } from '@angular/core';
import { ComponentFixture, TestModuleMetadata } from '@angular/core/testing';
import { AngularContext } from '../angular-context/angular-context';
/**
 * Provides the foundation for an opinionated pattern for component tests.
 *
 * - Includes all features from {@link AngularContext}
 * - Automatically creates your component at the beginning of `run()`.
 * - Sets up Angular to call `ngOnChanges()` like it would in production. This is not the case if you use the standard `TestBed.createComponent()` directly.
 * - Wraps your component in a dynamically created parent that you can easily style however you like.
 * - Lets you use {@link https://material.angular.io/cdk/test-harnesses/overview|component harnesses} in the `fakeAsync` zone, which is normally a challenge.
 * - Automatically disables animations.
 * - Automatically integrates {@link trimLeftoverStyles} to speed up your test suite.
 *
 * Why does the class name end with "Next"? This replaces the old `ComponentContext`, but it's a breaking change so this gives people some time to transition over. Eventually the old one will be removed and this will be renamed to `ComponentContext`.
 *
 * A very simple example:
 * ```ts
 * @Component({ template: 'Hello, {{name}}!' })
 * class GreeterComponent {
 *   @Input() name!: string;
 * }
 *
 * it('greets you by name', () => {
 *   const ctx = new ComponentContext(GreeterComponent);
 *   ctx.assignInputs({ name: 'World' });
 *   ctx.run(() => {
 *     expect(ctx.fixture.nativeElement.textContent).toBe('Hello, World!');
 *   });
 * });
 * ```
 *
 * A full example, with routing and a component harness. This is the full code for a tiny Angular app:
 * ```ts
 * /////////////////
 * // app-context.ts
 *
 * // To re-use your context setup, make a subclass of ComponentContext to
 * // import into any spec
 * class AppContext extends ComponentContext<AppComponent> {
 *   constructor() {
 *     super(AppComponent, {
 *       imports: [
 *         // This is your production `AppModule`. Make sure it exports `AppComponent`
 *         AppModule,
 *         // Import `routes` from your `app-routing.module.ts`
 *         RouterTestingModule.withRoutes(routes),
 *       ],
 *     });
 *   }
 * }
 *
 * ////////////////////////
 * // app.component.spec.ts
 *
 * describe('AppComponent', () => {
 *   let ctx: AppContext;
 *   beforeEach(() => {
 *     ctx = new AppContext();
 *   });
 *
 *   it('can navigate to the first page', () => {
 *     ctx.run(() => {
 *       ctx.getHarness(AppComponentHarness).navigateToFirstPage();
 *       expect(ctx.fixture.nativeElement.textContent).toContain(
 *         'First works!',
 *       );
 *     });
 *   });
 * });
 *
 * ///////////////////////////
 * // app.component.harness.ts
 *
 * // A simple component harness to demonstrate its integration with component contexts. Note that everything here is asynchronous, but is treated as synchronous in tests. This is a trick in AngularContexts so harnesses can be used in the fakeAsync zone.
 * class AppComponentHarness extends ComponentHarness {
 *   static hostSelector = 'app-root';
 *
 *   private getFirstPageLink = this.locatorFor('a');
 *
 *   async navigateToFirstPage(): Promise<void> {
 *     const link = await this.getFirstPageLink();
 *     await link.click();
 *   }
 * }
 *
 * /////////////////////
 * // first.component.ts
 *
 * // A minimal component for demonstration purposes
 * @Component({ template: '<p>First works!</p>' })
 * class FirstComponent {}
 *
 * ///////////////////
 * // app.component.ts
 *
 * // A minimal app component with routing for demonstration purposes
 * @Component({
 *   selector: 'app-root',
 *   template: `
 *     <a routerLink="/first-page">First Page</a>
 *     <router-outlet></router-outlet>
 *   `,
 * })
 * class AppComponent {}
 *
 * ////////////////////////
 * // app-routing.module.ts
 *
 * // exported for use in tests
 * const routes: Routes = [{ path: 'first-page', component: FirstComponent }];
 *
 * @NgModule({
 *   imports: [RouterModule.forRoot(routes)],
 *   exports: [RouterModule],
 * })
 * class AppRoutingModule {}
 *
 * ////////////////
 * // app.module.ts
 *
 * // A minimal app module. Notice the added export.
 * @NgModule({
 *   declarations: [AppComponent, FirstComponent],
 *   imports: [BrowserModule, BrowserAnimationsModule, AppRoutingModule],
 *   bootstrap: [AppComponent],
 *   exports: [AppComponent], // exported for use in tests
 * })
 * class AppModule {}
 * ```
 */
export declare class ComponentContext<T> extends AngularContext {
    /**
     * The {@link ComponentFixture} for a synthetic wrapper around your component. Available with the callback to `run()`.
     */
    fixture: ComponentFixture<unknown>;
    private componentType;
    private wrapperType;
    private inputProperties;
    private inputs;
    private wrapperStyles;
    /**
     * @param componentType `run()` will create a component of this type before running the rest of your test.
     * @param moduleMetadata passed along to [TestBed.configureTestingModule()]{@linkcode https://angular.io/api/core/testing/TestBed#configureTestingModule}. Automatically includes {@link NoopAnimationsModule}, in addition to those provided by {@link AngularContext}.
     * @param unboundInputs By default a synthetic parent component will be created that binds to all your component's inputs. Pass input names here that should NOT be bound. This is useful e.g. to test the default value of an input.
     */
    constructor(componentType: Type<T>, moduleMetadata?: TestModuleMetadata, unboundInputs?: Array<keyof T>);
    /**
     * Assign css styles to the div wrapping your component. Can be called before or during `run()`. Accepts an object with the same structure as the {@link https://angular.io/api/common/NgStyle|ngStyle directive}.
     *
     * ```ts
     * ctx.assignWrapperStyles({
     *   width: '400px',
     *   height: '600px',
     *   margin: '20px auto',
     *   border: '1px solid',
     * });
     * ```
     */
    assignWrapperStyles(styles: {
        [klass: string]: any;
    }): void;
    /**
     * Assign inputs passed into your component. Can be called before `run()` to set the initial inputs, or within `run()` to update them and trigger all the appropriate change detection and lifecycle hooks.
     */
    assignInputs(inputs: Partial<T>): void;
    /**
     * Use within `run()` to get your instantiated component that is on the page.
     */
    getComponentInstance(): T;
    /**
     * Constructs and initializes your component. Called during `run()` before it executes the rest of your test. Runs in the same `fakeAsync` zone as the rest of your test.
     */
    protected init(): void;
    /** @hidden */
    protected runChangeDetection(): void;
    /**
     * Performs any cleanup needed at the end of each test. This implementation destroys {@link fixture} and calls the super implementation.
     */
    protected cleanUp(): void;
    private isInitialized;
    private flushInputsToWrapper;
    private flushStylesToWrapper;
    private getWrapperComponentInstance;
}
